# Generated by roxygen2: do not edit by hand

S3method(arrange,duckspatial_df)
S3method(arrow::as_arrow_table,duckspatial_df)
S3method(arrow::as_record_batch_reader,duckspatial_df)
S3method(as_duckspatial_df,character)
S3method(as_duckspatial_df,data.frame)
S3method(as_duckspatial_df,duckspatial_df)
S3method(as_duckspatial_df,sf)
S3method(as_duckspatial_df,tbl_duckdb_connection)
S3method(as_duckspatial_df,tbl_lazy)
S3method(collect,duckspatial_df)
S3method(compute,duckspatial_df)
S3method(count,duckspatial_df)
S3method(ddbs_crs,character)
S3method(ddbs_crs,crs)
S3method(ddbs_crs,data.frame)
S3method(ddbs_crs,default)
S3method(ddbs_crs,duckdb_connection)
S3method(ddbs_crs,duckspatial_df)
S3method(ddbs_crs,numeric)
S3method(ddbs_crs,sf)
S3method(ddbs_crs,tbl_duckdb_connection)
S3method(distinct,duckspatial_df)
S3method(dplyr_reconstruct,duckspatial_df)
S3method(filter,duckspatial_df)
S3method(full_join,duckspatial_df)
S3method(geoarrow::infer_geoarrow_schema,duckspatial_df)
S3method(glimpse,duckspatial_df)
S3method(group_by,duckspatial_df)
S3method(head,duckspatial_df)
S3method(inner_join,duckspatial_df)
S3method(left_join,duckspatial_df)
S3method(mutate,duckspatial_df)
S3method(nanoarrow::as_nanoarrow_array_stream,duckspatial_df)
S3method(print,duckspatial_df)
S3method(rename,duckspatial_df)
S3method(right_join,duckspatial_df)
S3method(select,duckspatial_df)
S3method(slice,duckspatial_df)
S3method(st_as_sf,duckspatial_df)
S3method(st_bbox,duckspatial_df)
S3method(st_crs,duckspatial_df)
S3method(st_geometry,duckspatial_df)
S3method(summarise,duckspatial_df)
S3method(ungroup,duckspatial_df)
export(as_duckspatial_df)
export(ddbs_affine)
export(ddbs_area)
export(ddbs_as_geojson)
export(ddbs_as_hexwkb)
export(ddbs_as_mvt_geom)
export(ddbs_as_points)
export(ddbs_as_text)
export(ddbs_as_wkb)
export(ddbs_azimuth)
export(ddbs_bbox)
export(ddbs_boundary)
export(ddbs_buffer)
export(ddbs_build_area)
export(ddbs_centroid)
export(ddbs_collect)
export(ddbs_combine)
export(ddbs_compute)
export(ddbs_concave_hull)
export(ddbs_contains_properly)
export(ddbs_convex_hull)
export(ddbs_covered_by)
export(ddbs_covers)
export(ddbs_create_conn)
export(ddbs_create_schema)
export(ddbs_crop)
export(ddbs_crs)
export(ddbs_difference)
export(ddbs_dimension)
export(ddbs_disjoint)
export(ddbs_distance)
export(ddbs_drivers)
export(ddbs_drop_geometry)
export(ddbs_dump)
export(ddbs_endpoint)
export(ddbs_envelope)
export(ddbs_equals)
export(ddbs_extension_info)
export(ddbs_exterior_ring)
export(ddbs_filter)
export(ddbs_flip)
export(ddbs_flip_coordinates)
export(ddbs_force_2d)
export(ddbs_force_3d)
export(ddbs_force_4d)
export(ddbs_generate_points)
export(ddbs_geom_col)
export(ddbs_geom_from_geojson)
export(ddbs_geom_from_hexewkb)
export(ddbs_geom_from_hexwkb)
export(ddbs_geom_from_text)
export(ddbs_geom_from_wkb)
export(ddbs_geometry_type)
export(ddbs_get_ngeometries)
export(ddbs_get_ninterior_rings)
export(ddbs_get_npoints)
export(ddbs_get_resources)
export(ddbs_glimpse)
export(ddbs_has_m)
export(ddbs_has_z)
export(ddbs_install)
export(ddbs_interpolate_aw)
export(ddbs_intersection)
export(ddbs_intersection_agg)
export(ddbs_intersects)
export(ddbs_intersects_extent)
export(ddbs_is_closed)
export(ddbs_is_empty)
export(ddbs_is_ring)
export(ddbs_is_simple)
export(ddbs_is_valid)
export(ddbs_is_within_distance)
export(ddbs_join)
export(ddbs_length)
export(ddbs_line_endpoint)
export(ddbs_line_interpolate)
export(ddbs_line_locate_point)
export(ddbs_line_merge)
export(ddbs_line_node)
export(ddbs_line_startpoint)
export(ddbs_line_substring)
export(ddbs_list_tables)
export(ddbs_load)
export(ddbs_locate_along)
export(ddbs_locate_between)
export(ddbs_m)
export(ddbs_make_envelope)
export(ddbs_make_line)
export(ddbs_make_polygon)
export(ddbs_make_valid)
export(ddbs_maximum_inscribed_circle)
export(ddbs_minimum_rotated_rectangle)
export(ddbs_mmax)
export(ddbs_mmin)
export(ddbs_multi)
export(ddbs_normalize)
export(ddbs_open_dataset)
export(ddbs_options)
export(ddbs_overlaps)
export(ddbs_perimeter)
export(ddbs_point)
export(ddbs_polygonize)
export(ddbs_predicate)
export(ddbs_quadkey)
export(ddbs_read_meta)
export(ddbs_read_table)
export(ddbs_read_vector)
export(ddbs_reduce_precision)
export(ddbs_register_table)
export(ddbs_register_vector)
export(ddbs_remove_repeated_points)
export(ddbs_reverse)
export(ddbs_rotate)
export(ddbs_rotate_3d)
export(ddbs_scale)
export(ddbs_set_crs)
export(ddbs_set_resources)
export(ddbs_shear)
export(ddbs_shift)
export(ddbs_shortest_line)
export(ddbs_simplify)
export(ddbs_sitrep)
export(ddbs_startpoint)
export(ddbs_stop_conn)
export(ddbs_sym_difference)
export(ddbs_touches)
export(ddbs_transform)
export(ddbs_union)
export(ddbs_union_agg)
export(ddbs_vertices)
export(ddbs_voronoi)
export(ddbs_within)
export(ddbs_within_properly)
export(ddbs_write_dataset)
export(ddbs_write_mbtiles)
export(ddbs_write_table)
export(ddbs_write_vector)
export(ddbs_x)
export(ddbs_xmax)
export(ddbs_xmin)
export(ddbs_y)
export(ddbs_ymax)
export(ddbs_ymin)
export(ddbs_z)
export(ddbs_zmax)
export(ddbs_zmin)
export(is_duckspatial_df)
importFrom(dplyr,arrange)
importFrom(dplyr,collect)
importFrom(dplyr,compute)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,dplyr_reconstruct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,glimpse)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(lifecycle,deprecated)
importFrom(rlang,":=")
importFrom(sf,st_as_sf)
importFrom(sf,st_bbox)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(utils,head)
