#' @param id_x Character; optional name of the column in `x` whose values will
#' be used to name the list elements. If `NULL`, integer row numbers of `x` are used.
#' @param id_y Character; optional name of the column in `y` whose values will
#' replace the integer indices returned in each element of the list.
#' @param sparse A logical value. If `TRUE`, it returns a sparse table/list. If `FALSE`,
#' it returns a wide table/matrix.
