#' @param conn_x A `DBIConnection` object to a DuckDB database for the input `x`. 
#'        If `NULL` (default), it is resolved from `conn` or extracted from `x`.
#' @param conn_y A `DBIConnection` object to a DuckDB database for the input `y`. 
#'        If `NULL` (default), it is resolved from `conn` or extracted from `y`.
#' 
