#' @param by_feature Logical. If \code{TRUE}, the geometric operation is applied
#'        separately to each geometry. If \code{FALSE}, the geometric
#'        operation is applied to the data as a whole.
