% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_dimensions_ops.R
\name{ddbs_locate}
\alias{ddbs_locate}
\alias{ddbs_locate_along}
\alias{ddbs_locate_between}
\title{Locate geometries at specific M values}
\usage{
ddbs_locate_along(
  x,
  measure,
  offset = 0,
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)

ddbs_locate_between(
  x,
  start_measure,
  end_measure,
  offset = 0,
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{Input spatial data. Can be:
\itemize{
\item A \code{duckspatial_df} object (lazy spatial data frame via dbplyr)
\item An \code{sf} object
\item A \code{tbl_lazy} from dbplyr
\item A character string naming a table/view in \code{conn}
}
Data is returned from this object.}

\item{measure}{A numeric value specifying the M value at which to locate a
point along the geometry. Used only by \code{ddbs_locate_along}.}

\item{offset}{A numeric value specifying a lateral offset to apply
perpendicular to the line direction at the located point(s). Default is
\code{0} (no offset).}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
Can be set globally via \code{\link{ddbs_options}(mode = "...")} or
per-function via this argument. Per-function overrides global setting.}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}

\item{start_measure}{A numeric value specifying the lower bound of the M
range.}

\item{end_measure}{A numeric value specifying the upper bound of the M
range.}
}
\value{
Depends on the \code{mode} argument (or global preference set by \code{\link{ddbs_options}}):
\itemize{
\item \code{duckspatial} (default): A \code{duckspatial_df} (lazy spatial data frame) backed by dbplyr/DuckDB.
\item \code{sf}: An eagerly collected object in R memory, that will return the same data type as the
\code{sf} equivalent (e.g. \code{sf} or \code{units} vector).
}
When \code{name} is provided, the result is also written as a table or view in DuckDB and the function returns \code{TRUE} (invisibly).
}
\description{
Return the geometries at a specific M values or range of M values.
}
\details{
\itemize{
\item \code{ddbs_locate_along()}: returns a point or multi-point, containing the point(s)
at the geometry with the given measure
\item \code{ddbs_locate_between()}: returns a geometry or geometry collection created by
filtering and interpolating vertices within a range of "M" values
}
}
\examples{
\dontrun{
## load package
library(duckspatial)

## read data (must contain M-enabled linestring geometries)
rivers_ddbs <- ddbs_open_dataset(
  system.file("spatial/rivers.geojson",
  package = "duckspatial")
)

## Calculate the length of the rivers
rivers_agg_ddbs <- rivers_ddbs |> 
  ddbs_union_agg("RIVER_NAME") |> 
  ddbs_length()

## Add M dimension to the rivers
rivers_m_ddbs <- rivers_agg_ddbs |> 
  ddbs_force_3d("length", dim = "M")

## Locate rivers with M between 10000 and 20000
ddbs_locate_between(rivers_m_ddbs, 10000, 20000)

}
}
