% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duckspatial_df_sf_methods.R
\name{convert_wkb_to_sfc}
\alias{convert_wkb_to_sfc}
\title{Convert WKB data to sfc}
\usage{
convert_wkb_to_sfc(geom_data, crs)
}
\description{
Convert WKB data to sfc
}
\keyword{internal}
