% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_ops_unary_line.R
\name{ddbs_line_interpolate}
\alias{ddbs_line_interpolate}
\title{Interpolates a point or points along a line geometry}
\usage{
ddbs_line_interpolate(
  x,
  fraction = 0.5,
  intervals = FALSE,
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{Input spatial data. Can be:
\itemize{
\item A \code{duckspatial_df} object (lazy spatial data frame via dbplyr)
\item An \code{sf} object
\item A \code{tbl_lazy} from dbplyr
\item A character string naming a table/view in \code{conn}
}
Data is returned from this object.}

\item{fraction}{a numeric value between 0 and 1. When
\code{intervals = FALSE}, specifies the position along the line to
interpolate, where \code{0} is the start and \code{1} is the end.
When \code{intervals = TRUE}, specifies the spacing between interpolated
points as a proportion of the total line length. Defaults to \code{0.5}.}

\item{intervals}{a logical value. If \code{FALSE} (default), returns a
single \code{POINT} at the position given by \code{fraction}. If
\code{TRUE}, returns a \code{MULTIPOINT} of equally-spaced points along
the line at intervals defined by \code{fraction}.}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
Can be set globally via \code{\link{ddbs_options}(mode = "...")} or
per-function via this argument. Per-function overrides global setting.}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
Depends on the \code{mode} argument (or global preference set by \code{\link{ddbs_options}}):
\itemize{
\item \code{duckspatial} (default): A \code{duckspatial_df} (lazy spatial data frame) backed by dbplyr/DuckDB.
\item \code{sf}: An eagerly collected object in R memory, that will return the same data type as the
\code{sf} equivalent (e.g. \code{sf} or \code{units} vector).
}
When \code{name} is provided, the result is also written as a table or view in DuckDB and the function returns \code{TRUE} (invisibly).
}
\description{
Returns either a single point at a specified position along a line, or
multiple equally-spaced points along a line, depending on the value of
\code{intervals}. When \code{intervals = FALSE}, this wraps
\code{ST_LineInterpolatePoint}; when \code{intervals = TRUE}, it wraps
\code{ST_LineInterpolatePoints}.
}
\examples{
\dontrun{
## load package
library(duckspatial)

## read data
rivers_ddbs <- ddbs_open_dataset(
  system.file("spatial/rivers.geojson",
  package = "duckspatial")
)

## return the midpoint of a line (default)
ddbs_line_interpolate(rivers_ddbs)

## return the point 25\% along the line
ddbs_line_interpolate(rivers_ddbs, fraction = 0.25)

## return equally-spaced points every 10\% of the line length
ddbs_line_interpolate(rivers_ddbs, fraction = 0.1, intervals = TRUE)

## return equally-spaced points every 50\% of the line length (i.e. midpoint and end)
ddbs_line_interpolate(rivers_ddbs, fraction = 0.5, intervals = TRUE)
}
}
