% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_options.R
\name{ddbs_options}
\alias{ddbs_options}
\title{Get or set global duckspatial options}
\usage{
ddbs_options(output_type = NULL, mode = NULL, duckdb_storage_version = NULL)
}
\arguments{
\item{output_type}{Character string. Controls the default return type for \link{ddbs_collect}.
Must be one of:
\itemize{
\item \code{"sf"} (default): Eagerly collected \code{sf} object (in-memory).
\item \code{"tibble"}: Eagerly collected \code{tibble} without geometry.
\item \code{"raw"}: Eagerly collected \code{tibble} with geometry as raw WKB bytes.
\item \code{"geoarrow"}: Eagerly collected \code{tibble} with geometry as \code{geoarrow_vctr}.
}
If \code{NULL} (the default), the existing option is not changed.}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
If \code{NULL} (the default), the existing option is not changed.}

\item{duckdb_storage_version}{Character. The default DuckDB storage compatibility version
for newly created database files. See \url{https://duckdb.org/docs/internals/storage}
for details.}
}
\value{
Invisibly returns a list containing the currently set options.
}
\description{
Get or set global duckspatial options
}
\examples{
\dontrun{
# Set default mode to geoarrow
ddbs_options(mode = "geoarrow")

# Set default output to tibble
ddbs_options(output_type = "tibble")

# Check current settings
ddbs_options()
}
}
