% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_coordinates.R
\name{ddbs_xy}
\alias{ddbs_xy}
\alias{ddbs_x}
\alias{ddbs_y}
\alias{ddbs_m}
\alias{ddbs_z}
\title{Extract coordinates from geometries}
\usage{
ddbs_x(
  x,
  new_column = "X",
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)

ddbs_y(
  x,
  new_column = "Y",
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)

ddbs_m(
  x,
  new_column = "M",
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)

ddbs_z(
  x,
  new_column = "Z",
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{Input spatial data. Can be:
\itemize{
\item A \code{duckspatial_df} object (lazy spatial data frame via dbplyr)
\item An \code{sf} object
\item A \code{tbl_lazy} from dbplyr
\item A character string naming a table/view in \code{conn}
}
Data is returned from this object.}

\item{new_column}{Name of the new column to store the extracted coordinate.
Defaults to \code{"X"}, \code{"Y"}, \code{"M"}, or \code{"Z"}.}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
Can be set globally via \code{\link{ddbs_options}(mode = "...")} or
per-function via this argument. Per-function overrides global setting.}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
Depends on the \code{mode} argument (or global preference set by \code{\link{ddbs_options}}):
\itemize{
\item \code{duckspatial} (default): A \code{duckspatial_df} (lazy spatial data frame) backed by dbplyr/DuckDB.
\item \code{sf}: An eagerly collected object in R memory, that will return the same data type as the
\code{sf} equivalent (e.g. \code{sf} or \code{units} vector).
}
When \code{name} is provided, the result is also written as a table or view in DuckDB and the function returns \code{TRUE} (invisibly).
}
\description{
Extracts the X, Y, M, or Z coordinates from \code{POINT} geometries
}
\details{
\itemize{
\item \code{ddbs_x()}: Extracts the X coordinate (longitude).
\item \code{ddbs_y()}: Extracts the Y coordinate (latitude).
\item \code{ddbs_m()}: Extracts the M coordinate (measure).
\item \code{ddbs_z()}: Extracts the Z coordinate (elevation).
}
}
\examples{
\dontrun{
## load package
library(duckspatial)

## read data
argentina_ddbs <- ddbs_open_dataset(
  system.file("spatial/argentina.geojson",
  package = "duckspatial")
)

## extract coordinates
ddbs_x(argentina_ddbs)
ddbs_y(argentina_ddbs)
}
}
