% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_not_exported.R
\name{get_conn_from_input}
\alias{get_conn_from_input}
\title{Get DuckDB connection from an object}
\usage{
get_conn_from_input(x)
}
\arguments{
\item{x}{A duckspatial_df, tbl_lazy, duckdb_connection, or NULL}
}
\value{
A duckdb_connection or NULL
}
\description{
Extracts the connection from duckspatial_df, tbl_lazy, or validates a direct connection.
}
\keyword{internal}
