% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_crs.R
\name{ddbs_set_crs}
\alias{ddbs_set_crs}
\title{Set the coordinate reference system of geometries}
\usage{
ddbs_set_crs(
  x,
  y,
  conn = NULL,
  name = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{x}{Input spatial data. Can be:
\itemize{
\item A \code{duckspatial_df} object (lazy spatial data frame via dbplyr)
\item An \code{sf} object
\item A \code{tbl_lazy} from dbplyr
\item A character string naming a table/view in \code{conn}
}
Data is returned from this object.}

\item{y}{Target CRS. Can be:
\itemize{
\item A character string with EPSG code (e.g., "EPSG:4326")
\item A \code{crs} object created with \link[sf:st_crs]{sf::st_crs}
}}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
Can be set globally via \code{\link{ddbs_options}(mode = "...")} or
per-function via this argument. Per-function overrides global setting.}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
Depends on the \code{mode} argument (or global preference set by \code{\link{ddbs_options}}):
\itemize{
\item \code{duckspatial} (default): A \code{duckspatial_df} (lazy spatial data frame) backed by dbplyr/DuckDB.
\item \code{sf}: An eagerly collected object in R memory, that will return the same data type as the
\code{sf} equivalent (e.g. \code{sf} or \code{units} vector).
}
When \code{name} is provided, the result is also written as a table or view in DuckDB and the function returns \code{TRUE} (invisibly).
}
\description{
Assigns or replaces the coordinate reference system (CRS) of geometries
without transforming their coordinates. This is useful when the CRS is
missing or incorrectly defined.
}
\examples{
\dontrun{
## load package
library(duckspatial)
library(sf)

## read data
rivers_ddbs <- ddbs_open_dataset(
  system.file("spatial/rivers.geojson",
  package = "duckspatial")
)

## Remove CRS
rivers_no_crs_ddbs <- ddbs_set_crs(rivers_ddbs, st_crs(NA))

## Set the CRS back
ddbs_set_crs(rivers_no_crs_ddbs, "EPSG:3035")
}
}
