% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_not_exported.R
\name{import_view_to_connection}
\alias{import_view_to_connection}
\title{Import a view/table from one connection to another}
\usage{
import_view_to_connection(
  target_conn,
  source_conn,
  source_object,
  target_name = NULL,
  quiet = FALSE
)
}
\arguments{
\item{target_conn}{Target DuckDB connection}

\item{source_conn}{Source DuckDB connection}

\item{source_object}{duckspatial_df, tbl_lazy, or tbl_duckdb_connection from source_conn}

\item{target_name}{Name for view in target connection (auto-generated if NULL)}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
List with imported view name and cleanup function
}
\description{
Enables cross-connection operations by importing views using one of three strategies.
}
\keyword{internal}
