% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ddbs_extent_funs.R
\name{ddbs_make_envelope}
\alias{ddbs_make_envelope}
\title{Create a rectangular polygon from bounding coordinates}
\usage{
ddbs_make_envelope(
  xmin,
  ymin,
  xmax,
  ymax,
  crs = "EPSG:4326",
  name = NULL,
  conn = NULL,
  mode = NULL,
  overwrite = FALSE,
  quiet = FALSE
)
}
\arguments{
\item{xmin}{A numeric value for the minimum X (longitude) coordinate.}

\item{ymin}{A numeric value for the minimum Y (latitude) coordinate.}

\item{xmax}{A numeric value for the maximum X (longitude) coordinate.}

\item{ymax}{A numeric value for the maximum Y (latitude) coordinate.}

\item{crs}{A character string specifying the coordinate reference system
of the output geometry. Default is \code{"EPSG:4326"}.}

\item{name}{A character string of length one specifying the name of the table,
or a character string of length two specifying the schema and table
names. If \code{NULL} (the default), the function returns the result as an
\code{sf} object}

\item{conn}{A connection object to a DuckDB database. If \code{NULL}, the function
runs on a temporary DuckDB database.}

\item{mode}{Character. Controls the return type. Options:
\itemize{
\item \code{"duckspatial"} (default): Lazy spatial data frame backed by dbplyr/DuckDB
\item \code{"sf"}: Eagerly collected sf object (uses memory)
}
Can be set globally via \code{\link{ddbs_options}(mode = "...")} or
per-function via this argument. Per-function overrides global setting.}

\item{overwrite}{Boolean. whether to overwrite the existing table if it exists. Defaults
to \code{FALSE}. This argument is ignored when \code{name} is \code{NULL}.}

\item{quiet}{A logical value. If \code{TRUE}, suppresses any informational messages.
Defaults to \code{FALSE}.}
}
\value{
Depends on the \code{mode} argument (or global preference set by \code{\link{ddbs_options}}):
\itemize{
\item \code{duckspatial} (default): A \code{duckspatial_df} (lazy spatial data frame) backed by dbplyr/DuckDB.
\item \code{sf}: An eagerly collected object in R memory, that will return the same data type as the
\code{sf} equivalent (e.g. \code{sf} or \code{units} vector).
}
When \code{name} is provided, the result is also written as a table or view in DuckDB and the function returns \code{TRUE} (invisibly).
}
\description{
Creates a rectangular POLYGON geometry from four bounding coordinates.
}
\examples{
\dontrun{
## load package
library(duckspatial)

## without storing in duckdb
finland_bbox_ddbs <- ddbs_make_envelope(
  xmin = 19.1, ymin = 59.7,
  xmax = 31.6, ymax = 70.1
)
}
}
