% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_not_exported.R
\name{dbConnCheck}
\alias{dbConnCheck}
\title{Check if a supported DuckDB connection}
\usage{
dbConnCheck(conn)
}
\arguments{
\item{conn}{A \code{DBIConnection} object to a DuckDB database}
}
\value{
TRUE (invisibly) for successful import
}
\description{
Check if a supported DuckDB connection
}
\keyword{internal}
